/*
	This file is part of cpso.

	cpso is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	cpso is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with cpso.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2010 Lucas Hermann Negri
*/


#include "cpso.h"
#define rnd() ( rand() / (double)RAND_MAX )

static float rnd_range(double min, double max)
{
	double r = rand() / (double)RAND_MAX;
	return min + r * (max - min);
}

static void copy_particle(CpsoState* st, int from, int to, double fit)
{
    int d;
    for(d = 0; d < st->n_dim; ++d)
        st->b_pos[to][d] = st->space[from][d];

    st->b_fit[to] = fit;
}

CpsoState* cpso_new(int n_dim, int n_pop)
{
    CpsoState* st = malloc( sizeof(CpsoState) );
    if(!st) return NULL;
    
    st->space = NULL;
    st->b_fit = NULL;
    st->limits = NULL;
    
    st->n_dim = n_dim;
    st->n_pop = n_pop;
    
    int len = n_pop + 1;
    st->space = malloc( sizeof(double*) * len * 3 );
    if(!st->space)      { cpso_free(st); return NULL; }
    
    st->velo  = &st->space[len    ];
    st->b_pos = &st->space[len * 2];
    
    st->b_fit = malloc( sizeof(double) * (n_pop + 1) );
    if(!st->b_fit)      { cpso_free(st); return NULL; }
    
    st->limits= malloc( sizeof(double) * n_dim * (2 + len * 3) );
    if(!st->limits)     { cpso_free(st); return NULL; }
    
    int p;
    for(p = 0; p <= n_pop; ++p)
    {
        st->space[p] = &st->limits[n_dim * (2 + 3 * p)];
        st->velo[p]  = &st->limits[n_dim * (3 + 3 * p)];
        st->b_pos[p] = &st->limits[n_dim * (4 + 3 * p)];
    }
    
    st->c1 = st->c2 = 2;
    st->w  = 0.5;
    st->r  = 0.01;
    st->eval = st-> ud = NULL;
    
    return st;
}

void cpso_free(CpsoState* st)
{
    free(st->limits);
    free(st->b_fit);
    free(st->space);
    free(st);
}

void cpso_set_limits(CpsoState* st, int dim, double min, double max)
{
    int of = dim * 2;
    st->limits[of  ] = min;
    st->limits[of+1] = max;
}

void cpso_set_params(CpsoState* st, double c1, double c2, double w, double r)
{
    if(c1 != -1) st->c1 = c1;
    if(c2 != -1) st->c2 = c2;
    if(w  != -1) st->w = w;
    if(r  != -1) st->r = r;
}

void cpso_set_eval(CpsoState* st, double (eval)(CpsoState*, const double*), void* ud)
{
    st->eval = eval;
    st->ud   = ud;
}

void cpso_new_particle(CpsoState* st, int p, char clear)
{
    int d;
    for(d = 0; d < st->n_dim; ++d)
    {
        int of     = d * 2;
        double val = rnd_range(st->limits[of], st->limits[of + 1]);
                
        st->space[p][d] = val;
        if(clear) st->b_pos[p][d] = val;
    }
    
    if(clear)
        st->b_fit[p] = st->eval(st, st->space[p]);
}

void cpso_init(CpsoState* st)
{
    int i = 0;
    for(i = 0; i < st->n_pop; ++i)
        cpso_new_particle(st, i, 1);
    
    double b_init = st->eval(st, st->space[0]);
    copy_particle(st, 0, st->n_pop, b_init);
}

void cpso_step(CpsoState* st)
{
    double max_fit = -1, max_idx = -1;
    int n_pop = st->n_pop, n_dim = st->n_dim;
    
    int p, d;
    for(p = 0; p < n_pop; ++p)
    {
        /* particle replacement */
        if(rnd() < st->r)
            cpso_new_particle(st, p, 0);        
        
        /* fitness evalutation */
        double fit = st->eval(st, st->space[p]);
        
        /* update the local best */
        if(fit > st->b_fit[p])
            copy_particle(st, p, p, fit);
            
        /* keep track of the global best */
        if(fit > max_fit || max_idx == -1)
        {
            max_idx = p;
            max_fit = fit;
        }
    }
    
    /* update the global best */
    if(max_fit > st->b_fit[n_pop])
        copy_particle(st, max_idx, n_pop, max_fit);
    
    /* update the particle velocity and position */
    double c1 = st->c1, c2 = st->c2;
    for(p = 0; p < n_pop; ++p)
    {
        for(d = 0; d < n_dim; ++d)
        {
            st->velo[p][d] = st->velo[p][d] * st->w +                   /* inertia   */
                c1 * rnd() * (st->b_pos[p][d]     - st->space[p][d]) +  /* cognitive */
                c2 * rnd() * (st->b_pos[n_pop][d] - st->space[p][d]);   /* social    */
            
            double new_pos = st->space[p][d] + st->velo[p][d];
            
            /* respect the limits */
            int of = d * 2;
            
            if(new_pos < st->limits[of])
                new_pos = st->limits[of];
            else if(new_pos > st->limits[of + 1])
                new_pos = st->limits[of + 1];
        
            st->space[p][d] = new_pos;
        }
    }
}

int cpso_run(CpsoState* st, int max_iters, double max_fit)
{
    int i, n_pop = st->n_pop;
    for(i = 0; i < max_iters; ++i)
    {
        cpso_step(st);
        if( st->b_fit[n_pop] >= max_fit ) break;
    }
    
    return i;
}

double cpso_get_best(CpsoState* st, int p, const double** out)
{
    if(p < 0 || p > st->n_pop)  p = st->n_pop;
    if(out)                     *out = st->b_pos[p];
    
    return st->b_fit[p];
}
