/*
	This file is part of cpso.

	cpso is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	cpso is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with cpso.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2010 Lucas Hermann Negri
*/

#include <stdlib.h>

/* Main structure, holds the algorithm state. Shouldn't be used directly. */
typedef struct CpsoState
{
    double**    space;      /* current particle position */
    double**    velo;       /* current particle velocity */
    double**    b_pos;      /* best particle position (last is the global best) */
    double*     b_fit;      /* best particle fitness  (last is the global best) */
    double*     limits;     /* limits (min,max) of the dimensions */
    double      (*eval)(struct CpsoState*, const double* part);
                            /* fitness evaluation function */
    void*       ud;         /* userdata */
    double      c1;         /* cognitive factor */
    double      c2;         /* social factor */
    double      w;          /* inertia */
    double      r;          /* replacement probability */
    int         n_dim;      /* number of dimensions */
    int         n_pop;      /* population size */
} CpsoState;

/* Constructor */
CpsoState* cpso_new(int n_dim, int n_pop);

/* Destructor */
void cpso_free(CpsoState* st);

/* Sets the limits of a dimension */
void cpso_set_limits(CpsoState* st, int dim, double min, double max);

/* Sets the algorithm parameters */
void cpso_set_params(CpsoState* st, double c1, double c2, double w, double r);

/* Sets fitness evaluation function */
void cpso_set_eval(CpsoState* st, double (eval)(CpsoState*, const double*), void* ud);

/* Initializes one particle */
void cpso_new_particle(CpsoState* st, int p, char clear);

/* Initializes all the state particles */
void cpso_init(CpsoState* st);

/* Runs one iteration of the algorithm */
void cpso_step(CpsoState* st);

/* Runs the algorithm until reach a certain number of iterations or a certain fitness */
int cpso_run(CpsoState* st, int max_iters, double max_fit);

/* Gets the best position and fitness of a particle */
double cpso_get_best(CpsoState* st, int p, const double** out);
